require File.expand_path(File.dirname(__FILE__) + "/../spec_helper")

describe Emvoicer::Services::Base do
  let(:service) do
    Emvoicer::Services::Base.new
  end

  before(:each) do
    service.stubs(:headers).returns({ "Accept" => "application/xml" })
    service.stubs(:invoices_url).returns("http://example.com/invoices")
    service.stubs(:client_url).with(1).returns("http://example.com/clients/1")
  end

  describe ".request(url, params, callback)" do
    it "issues GET request wih given parameters" do
      stub_request(:get, "http://example.com/?status=open").
        with(:headers => { "Accept" => "application/xml"}).
        to_return(:status => 200, :body => "", :headers => {})
      
      EM.run_block do
        service.request("http://example.com",
                        { :status => "open" },
                        proc {})
      end

      WebMock.should have_requested(:get, "http://example.com").with(
        :query => { :status => "open" },
        :headers => { "Accept" => "application/xml" }
      )
    end

    context "on successfull request" do
      it "calls provided callback" do
        stub_request(:get, "http://example.com/?status=open").
          with(:headers => { "Accept" => "application/xml"}).
          to_return(:status => 200, :body => "", :headers => {})
        
        called = false

        EM.run_block do
          service.request("http://example.com",
                          { :status => "open" },
                          proc { called = true })
        end

        called.should be_true
      end
    end
  end

  describe ".invoices(params, callback)" do
    let(:response) { "Response" }
    let(:invoice) { "Invoice" }

    it "issues a get request with invoices_url and provided params" do
      service.expects(:request)

      service.invoices({ :status => "open" }, proc {})
    end

    it "calls callback with invoices collection" do
      stub_request(:get, "http://example.com/invoices?status=open").
        with(:headers => {"Accept"=>"application/xml"}).
        to_return(:status => 200, :body => response, :headers => {})
      
      service.expects(:handle_invoices_source).with(response).returns([invoice])

      called = false

      EM.run_block do
        service.invoices({ :status => "open" }, proc { |invoices| called = true; invoices.should eq([invoice]) })
      end

      called.should be_true
    end
  end

  describe ".handle_invoices_source(source)" do
    let(:source) { "Source" }
    let(:node) { "Node" }
    let(:invoice) { "Invoice" }

    it "generates nodes from source" do
      service.class.expects(:extract_invoice_nodes).with(source).returns([node])
      service.handle_invoices_source(source)
    end

    it "returns generated invoices from nodes" do
      service.class.stubs(:extract_invoice_nodes).with(source).returns([node])
      service.expects(:generate_invoice).with(node).returns(invoice)
      service.handle_invoices_source(source).should eq([invoice])
    end
  end

  describe ".generate_invoice(node)" do
    let(:node) { "Node" }
    let(:attributes) do
      {
        :id => 1,
        :number => "100001",
        :total_amount => 20.0,
        :due_amount => 10.0,
        :issued_on => Date.new(2011, 3, 13),
        :due_on => Date.new(2011, 3, 28),
        :currency => "USD",
        :client_id => 1
      }
    end

    it "parses given node" do
      service.class.expects(:parse_invoice).with(node).returns(attributes)
      service.generate_invoice(node)
    end

    it "creates Invoice instance with service and node attributes" do
      service.class.stubs(:parse_invoice).with(node).returns(attributes)
      Emvoicer::Invoice.expects(:new).with(service, attributes)
      service.generate_invoice(node)
    end
  end

  describe ".client(id, callback)" do
    let(:response) { "Response" }
    let(:client) { "Client" }

    it "issues a get request with client_url and provided id" do
      service.expects(:request)

      service.client(1, proc {})
    end

    it "calls callback with client" do
      stub_request(:get, "http://example.com/clients/1").
        with(:headers => {"Accept"=>"application/xml"}).
        to_return(:status => 200, :body => response, :headers => {})

      service.expects(:handle_client_source).with(response).returns([client])

      called = false

      EM.run_block do
        service.client(1, proc { |invoices| called = true; client.should eq(client) })
      end

      called.should be_true
    end
  end

  describe ".handle_client_source(source)" do
    let(:source) { "Source" }
    let(:node) { "Node" }
    let(:client) { "Client" }

    it "generates node from source" do
      service.class.expects(:extract_client_node).with(source).returns(node)
      service.handle_client_source(source)
    end

    it "returns generated client from node" do
      service.class.stubs(:extract_client_node).with(source).returns(node)
      service.expects(:generate_client).with(node).returns(client)
      service.handle_client_source(source).should eq(client)
    end
  end

  describe ".generate_client(node)" do
    let(:node) { "Node" }
    let(:attributes) do
      {
        :id => 1,
        :name => "Haute Haute Heat",
        :address_1 => "1234 Main Street",
        :address_2 => "Suite 123",
        :city => "Anytown",
        :zip_code => "70123",
        :country => "US",
        :phone => "123-555-1212"
      }
    end

    it "parses given node" do
      service.class.expects(:parse_client).with(node).returns(attributes)
      service.generate_client(node)
    end

    it "creates Client instance with service and node attributes" do
      service.class.stubs(:parse_client).with(node).returns(attributes)
      Emvoicer::Client.expects(:new).with(service, attributes)
      service.generate_client(node)
    end
  end
end
